
require 'pf_botnet'
require 'escape'
require 'yaml'


class Armanoide < PfBot
  attr_accessor :announces_mode, :announces_registered_nicks

  def initialize(jidstr, passwd, channel_jid, channel_passwd, bot)
    super(jidstr, passwd, channel_jid, channel_passwd)
    @bot = bot

    @announces_mode = :public
    @announces_registered_nicks = []
  end

  def process_unknown(time, nick, cmdline)
    return if time    # ignore backlog

    # use BotConfig.register instead
    irc_channel = "#Proformatique-bots"

    type = cmdline.shift.upcase
  
    case type
    when "ANNOUNCE"
      msg = "[ANNOUNCE] #{nick}> " + cmdline.join(" ")
      case @announces_mode
      when :public
        @bot.say irc_channel, msg
      when :registered
        @announces_registered_nicks.each {|nick| @bot.say nick, msg }
      end
    end
  end

end

class PfBotNetPlugin < Plugin

  def initialize
    super

    # use BotConfig.register instead
    begin
      bot_conf = YAML.load(File.read("/etc/pfbotnet/armanoide.conf"))
    rescue
      STDERR.puts "Cannot load bot config"
      exit 1
    end
    begin
      channel_conf = YAML.load(File.read("/etc/pfbotnet/channel.conf"))
    rescue
      STDERR.puts "Cannot load channel config"
      exit 1
    end

    begin
      @pfb = Armanoide.new(bot_conf['bot_jid'], bot_conf['bot_passwd'], channel_conf['channel_jid'], channel_conf['channel_passwd'], @bot)
      @pfb.away(bot_conf['bot_away_msg'])
    rescue
      raise "Problem with the messaging system: " + $!
      @pfb = nil
    end
  end
  
  def help(plugin, topic="")
    t_parts = topic.gsub(/[ ]+/, " ").split(" ")

    case plugin
    when "proformatique"
      "Proformatique Services - plugins: dak dhcpdisco pak pfbotnet sys"
    when "pfbotnet"
      case t_parts[0]
      when nil
        "Proformatique Botnet Administration - topic: status botlist announces"
      when "status"
        "Retrieve botnet general status [NOTIMPL]"
      when "botlist"
        "List bots joined in the private partyline"
      when "announces"
        "dak botnet announces mode [PUBLIC|REGISTERED|OFF] -> Select relaying botnet announces to the channel (PUBLIC), to users having registered (REGISTERED), to nobody (OFF), or display current mode if no parameter given. dak botnet announces [UN]REGISTER -> register or unregister to announces watch. dak botnet announces LIST-REGISTERED -> display registered nicks"
      end
    when "dak"
      case t_parts[0]
      when nil
        "DAK - topics: scan info ftpmaster"
      when "scan"
        "Scan for packages in the upload queue, and install ACCEPTED packages"
      when "info"
        case t_parts[1]
	when nil
          "Display various information - topics: new lver"
	when "new"
	  "Display NEW queue summary"
	when "lver"
	  "dak info lver <pkg> <arch> [<suite>] -> Display latest <pkg> version for architecture <arch> (limited to suite <suite> if specified)"
	end
      when "ftpmaster"
        case t_parts[1]
	when nil
          "DAK Management - topics: accept reject force-install"
	when "accept"
	  "dak ftpmaster accept <pkgid> -> Accept NEW package with id <pkgid> in the repository (see 'dak info new' output to get <pkgid>)"
	when "reject"
	  "dak ftpmaster reject <pkgid> [<reason>] -> Reject NEW package with id <pkgid> in the repository, with an optional <reason> (see 'dak info new' output to get <pkgid>)"
	when "force-install"
	  "dak ftpmaster force-install -> Force installation of newly ACCEPTED packages, even if not necessary"
	end
      end
    when "pak"
      case t_parts[0]
      when nil
        "PAK - topics: scan info ftpmaster"
      when "scan"
        "Scan for pending packages to be installed"
      when "info"
        case t_parts[1]
	when nil
	  "Display various information - topics: list lver"
	when "list"
	  "Display profiles/tags/suites list"
	when "lver"
	  "pak info lver <package> <arch> <profile> [<tag>] -> Display last version of the <package> for architecture <arch> and each suite of the specific <profile> (for the subrepository <tag> if this <profile> is split)"
	end
      when "ftpmaster"
        case t_parts[1]
	when nil
	  "PAK Management - topics: force-install"
	when "force-install"
	  "pak ftpmaster force-install <profile> [<tag>] -> Force package lists regeneration, even if not necessary, for a specific <profile> (for the subrepository <tag> if this <profile> is split)"
	end
      end
    when "sys"
      case t_parts[0]
      when nil
        "SYS - topics: push-ssh-authkeys"
      when "push-ssh-authkeys"
        "push-ssh-authkeys [<user>] -> regenerate '~/.ssh/authorized_keys' on all machines, for all users or limited to <user> if specified"
      end
    when "dhcpdisco"
      case t_parts[0]
      when nil
        "DHCPDISCO - topics: list clear announces"
      when "list"
        "dhcpdisco [@<node>] list -> Display list of discovered rogue servers"
      when "clear"
        "dhcpdisco [@<node>] clear -> Clear list of discovered rogue servers"
      when "announces"
        "dhcpdisco [@<node>] announces <ON|OFF> -> activate/deactivate DHCP log messages announcement"
      end
    else
      usage()
    end
  end

  def cmd_subsys_relay(subsys, m, params)
    if @pfb.nil?
      m.reply "Link with the botnet is broken !"
      return
    end

    @pfb.send_command(subsys + " " + Escape.shell_command(params).to_s) { |nick, result, extra|
      msg_prefix = "#{subsys}[#{nick}]> "
      case result
      when "DONE", "FAILED"
        msg = msg_prefix + "command " + result
	msg += " (#{extra})" unless extra.nil?
	m.reply msg
      when "RESULT:"
        m.reply msg_prefix + extra
      end
    }
  end

  def privmsg(m)
    if m.params.nil?
      m.reply "For help about Proformatique plugins, type: help proformatique"
      return
    end

    cmdline = Shellwords.shellwords(m.params)

    subsys = m.plugin.upcase
    case subsys
    when "PFBOTNET"
      case cmdline.shift.upcase
      when "STATUS"
        return m.reply "Not yet implemented"
      when "BOTLIST"
        list = @pfb.chan_members
        return m.reply "#{list.size} connected bots: " + list.join(" ")
      when "ANNOUNCES"
        subcmd = cmdline.shift
	unless subcmd. nil?
	  subcmd.upcase!
	  case subcmd
	  when "MODE"
	    return unless @bot.auth.allow?("botnetmaster", m.source, m.replyto)
	    flag =  cmdline.shift
	    if flag.nil?
	      return m.reply "Announces mode is " + @pfb.announces_mode.to_s.upcase
	    else
	      flag.upcase!
	      if ["PUBLIC", "REGISTERED", "OFF"].include? flag
	        if @bot.auth.allow?("botnetmaster", m.source, m.replyto)
	          @pfb.announces_mode = flag.downcase.to_sym
		  m.reply "Announces mode is now " + flag
		end
		return
	      end
	    end
	  when "REGISTER"
	    @pfb.announces_registered_nicks << m.replyto
	    return m.reply "You are now registered to announces watch"
	  when "UNREGISTER"
	    @pfb.announces_registered_nicks.delete(m.replyto)
	    return m.reply "You are now unregistered to announces watch"
	  when "LIST-REGISTERED"
	    m.reply "Nicks registered to announces: " + @pfb.announces_registered_nicks.join(", ") if @bot.auth.allow?("botnetmaster", m.source, m.replyto)
	    return
	  end
	end
      end
    when "DAK"
      case cmdline.first.upcase
      when "SCAN", "INFO"
        return cmd_subsys_relay(subsys, m, cmdline)
      when "FTPMASTER"
        return cmd_subsys_relay(subsys, m, cmdline) if @bot.auth.allow?("ftpmaster", m.source, m.replyto)
      end
    when "PAK"
      case cmdline.first.upcase
      when "SCAN", "INFO"
        return cmd_subsys_relay(subsys, m, cmdline)
      when "FTPMASTER"
        return cmd_subsys_relay(subsys, m, cmdline) if @bot.auth.allow?("ftpmaster", m.source, m.replyto)
      end
    when "SYS"
      case cmdline.first.upcase
      when "PUSH-SSH-AUTHKEYS"
        return cmd_subsys_relay(subsys, m, cmdline)
      end
    when "DHCPDISCO"
      cmd = ((cmdline[0][0..0] == "@") ? cmdline[1] : cmdline[0]).upcase

      case cmd
      when "LIST"
        return cmd_subsys_relay(subsys, m, cmdline)
      when "CLEAR"
        return cmd_subsys_relay(subsys, m, cmdline)
      when "ANNOUNCES"
        return cmd_subsys_relay(subsys, m, cmdline)
      end
    end

    m.reply "command not recognized"
  end

  def cleanup
    @pfb.quit
  end

end

plugin = PfBotNetPlugin.new
plugin.register("proformatique")
plugin.register("pfbotnet")
plugin.register("dak")
plugin.register("pak")
plugin.register("sys")
plugin.register("dhcpdisco")

